#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Collection of provider modules.

File: __init__.py
Author: huxuan
Email: i(at)huxuan.org
"""
from slothstock.providers.xueqiu import XueQiu

__all__ = [
    'XueQiu',
]
